import { LRParser } from '@lezer/lr';
import { LRLanguage, indentNodeProp, delimitedIndent, foldNodeProp, foldInside, LanguageSupport } from '@codemirror/language';
import { styleTags, tags } from '@lezer/highlight';
import { snippetCompletion, completeFromList } from '@codemirror/autocomplete';

// This file was generated by lezer-generator. You probably shouldn't edit it.
const parser = LRParser.deserialize({
  version: 14,
  states: "!WQYQPOOOnQPO'#CfOOQO'#Ck'#CkOOQO'#Cg'#CgQYQPOOOOQO,59Q,59QOuQPO,59QOOQO-E6e-E6eOOQO1G.l1G.l",
  stateData: "!P~O^OSPOS~ORQOSQOTQOUQOVQOXPO~OWTO~PYOWWO~PYOST~",
  goto: "w`PPPPPPPPPPagPPPqXQOPSUQSOQUPTVSUXROPSU",
  nodeNames: "⚠ LineComment Program Identifier String Name Boolean Number ) ( Application",
  maxTerm: 15,
  nodeProps: [
    ["openedBy", 8,"("],
    ["closedBy", 9,")"]
  ],
  skippedNodes: [0,1],
  repeatNodeCount: 1,
  tokenData: "%p~R`XY!TYZ!T]^!Tpq!Tst!fxy!tyz!y}!O#O!O!P#a!P!Q#o!Q![#U!c!v$^!v!w$f!w!}$^#R#S%Y#T#o%Y~!YS^~XY!TYZ!T]^!Tpq!T~!iQ#Y#Z!o#h#i!o~!tOU~~!yOX~~#OOW~~#RP!Q![#U~#ZQV~!O!P#a!Q![#U~#dP!Q![#g~#lPV~!Q![#g~#rP!P!Q#u~#zSP~OY#uZ;'S#u;'S;=`$W<%lO#u~$ZP;=`<%l#u~$cPT~#T#o$^~$kQT~!Q![$q#T#o$^~$tQ!O!P$z!Q![$q~$}P!Q![%Q~%VPS~!Q![%Q~%_RR~!Q![%h#R#S%Y#T#o%Y~%mPR~!Q![%h",
  tokenizers: [0],
  topRules: {"Program":[0,2]},
  tokenPrec: 44
});

/// A collection of JavaScript-related
/// [snippets](#autocomplete.snippet).
const snippets = [
    // compare
    snippetCompletion("(${value1} if ${condition} else ${value2})", {
        label: "if ... else",
        type: "keyword"
    }),
    // math
    snippetCompletion("dif(${tag})", {
        label: "dif",
        type: "keyword"
    }),
    snippetCompletion("max(${tag})", {
        label: "max",
        type: "keyword"
    }),
    snippetCompletion("min(${tag})", {
        label: "min",
        type: "keyword"
    }),
    snippetCompletion("avg(${tag})", {
        label: "avg",
        type: "keyword"
    }),
    snippetCompletion("abs(${param})", {
        label: "abs",
        type: "keyword"
    }),
    snippetCompletion("exp(${param})", {
        label: "exp",
        type: "keyword"
    }),
    snippetCompletion("log(${param})", {
        label: "log",
        type: "keyword"
    }),
    snippetCompletion("log10(${param})", {
        label: "log10",
        type: "keyword"
    }),
    snippetCompletion("sqrt(${param})", {
        label: "sqrt",
        type: "keyword"
    }),
    snippetCompletion("pow(${param}, ${degree})", {
        label: "pow",
        type: "keyword"
    }),
    snippetCompletion("hypot(${param}, ${param2})", {
        label: "hypot",
        type: "keyword"
    }),
    snippetCompletion("round(${param})", {
        label: "round",
        type: "keyword"
    }),
    snippetCompletion("floor(${param})", {
        label: "floor",
        type: "keyword"
    }),
    snippetCompletion("ceil(${param})", {
        label: "ceil",
        type: "keyword"
    }),
    // constants 
    snippetCompletion("Pi", {
        label: "Pi",
        type: "constant"
    }),
    snippetCompletion("E", {
        label: "E",
        type: "constant"
    }),
    // trigonometric
    snippetCompletion("sin(${param})", {
        label: "sin",
        type: "keyword"
    }),
    snippetCompletion("asin(${param})", {
        label: "asin",
        type: "keyword"
    }),
    snippetCompletion("sinh(${param})", {
        label: "sinh",
        type: "keyword"
    }),
    snippetCompletion("cos(${param})", {
        label: "cos",
        type: "keyword"
    }),
    snippetCompletion("acos(${param})", {
        label: "acos",
        type: "keyword"
    }),
    snippetCompletion("cosh(${param})", {
        label: "cosh",
        type: "keyword"
    }),
    snippetCompletion("tan(${param})", {
        label: "tan",
        type: "keyword"
    }),
    snippetCompletion("atan(${param})", {
        label: "atan",
        type: "keyword"
    }),
    snippetCompletion("tanh(${param})", {
        label: "tanh",
        type: "keyword"
    }),
];

const mathLanguage = LRLanguage.define({
    name: 'math',
    parser: parser.configure({
        props: [
            indentNodeProp.add({
                Application: delimitedIndent({ closing: ")", align: false })
            }),
            foldNodeProp.add({
                Application: foldInside
            }),
            styleTags({
                Identifier: tags.variableName,
                Boolean: tags.bool,
                Name: tags.name,
                String: tags.string,
                Number: tags.number,
                LineComment: tags.lineComment,
                "( )": tags.paren
            })
        ]
    }),
    languageData: {
        commentTokens: { line: ";" }
    }
});
function math() {
    return new LanguageSupport(mathLanguage, [
        mathLanguage.data.of({
            autocomplete: completeFromList(snippets)
        })
    ]);
}

export { math, mathLanguage };
